function ber = FourPam(SNR, mf)
    %Given SNR in dB, gets BER of the system
    %mf = 0--No match filter
    %mf = 1--Match filter
    
    N = 1000;
    fs = 50;

    %% Generate Transmitted Signal
    [txValue, bits, ~] = getTxValue(N);
    txPulse = getPulse(txValue, fs); 
    len = length(txPulse);

    %% Adding Noise To Transmitted Signal
    slin = 10^(SNR/10);
    sigVar = 5;
    noiseVar = sigVar/slin;
    noise = sqrt(noiseVar)*randn(len, 1);
    txSig = txPulse + noise;

    %% Receiving Signal--Matche Filtering
    %Then transmit over the channel
    %h is our matched filter
    h = ones(fs, 1);    %g(T-t) where T = 1;

    %Here we use matched filtering
    rxSig = conv(txSig, h)/fs;
   
    %Then we sample at T (aka delay by 1)
    if mf == 1
        y = rxSig((fs+1):fs:end);  
    else
        y = txSig(1:fs:end);
    end

    %% Decoding
    rxVal = y;

    for n = 1:length(y)
       rxVal(n) = sigDetect(y(n));
    end

    [rBits, ~] = recoverBits(rxVal);

    %% Calculate Bit Error Rate
    diff = rBits - bits;
    ber = numel(diff(diff ~= 0))/length(bits);
end