%% Copyright David Ding 2016-2020

%Main Script for PAM Demo
close all;
clear;
clc;

N = 50;
fs = 50;

%% Generate Transmitted Signal
[txValue, bits, syms] = getTxValue(N);
txPulse = getPulse(txValue, fs); 
len = length(txPulse);
plotTxPulse(txPulse, fs);

%% Adding Noise To Transmitted Signal
SNR = 5;    %db
slin = 10^(SNR/10);
sigVar = 5;
noiseVar = sigVar/slin;
noise = sqrt(noiseVar)*randn(len, 1);

txSig = txPulse + noise;
plotTxSig(txSig, fs);

%% Receiving Signal--Matched Filtering
% Then transmit over the channel
% h is our matched filter
h = ones(fs, 1);    %g(T-t) where T = 1;

% Here we use matched filtering
rxSig = conv(txSig, h)/fs;
figure;
plot((1:length(rxSig))/fs, rxSig);
xlabel('Time in Seconds (delayed by T = 1)');
ylabel('Value');
title('Received Signal After Match Filtering');

%Then we sample at T (aka delay by 1)
y = rxSig((fs+1):fs:end);   %y is the actual detected signal with matched filter
yRaw = txSig(1:fs:end); %yRaw is the detected sig with no matched filter

%We point out the sampled points on the y graph
figure;
plot((1:length(rxSig))/fs, rxSig);
hold on;
scatter((1:25), y, 'ro');
xlabel('Time in Seconds (delayed by T = 1)');
ylabel('Value');
title('Received Signal After Match Filtering, Showing Sample Points');

%% Decoding
rxVal = y;
rxValRaw = y;

for n = 1:length(y)
   rxVal(n) = sigDetect(y(n));
   rxValRaw(n) = sigDetect(yRaw(n));
end

[rBits, rSyms] = recoverBits(rxVal);
[rBitsRaw, ~] = recoverBits(rxValRaw);

%% Calculate Bit Error Rate
diff = rBits - bits;
ber = numel(diff(diff ~= 0))/length(bits);
berPerc = ber*100;
diff2 = rBitsRaw - bits;
ber2 = numel(diff2(diff2 ~= 0))/length(bits);
berPerc2 = ber2*100;

fprintf('Bit Error Rate With Match Filtering: %f%%\n', berPerc);
fprintf('Bit Error Rate With No Match Filtering: %f%%\n', berPerc2);